BEGIN { require 5.005 }

use strict;

my $NAME     = 'Lingua::Stem::Snowball';

my @snowballfiles = ( 'api.h', 'api.c', 'header.h', 'utilities.c' );

die "Please, add to enviroment SNOWBALL_SRC. Exiting...\n"
	if ( !exists $ENV{SNOWBALL_SRC} );
die "There is no $ENV{SNOWBALL_SRC}. Exiting...\n"
	if ( ! -d $ENV{SNOWBALL_SRC} );
foreach my $file ( @snowballfiles ) {
	die "Your Snowball is not full: there is no file $ENV{SNOWBALL_SRC}/q/$file. Exiting...\n"
		if ( ! -s "$ENV{SNOWBALL_SRC}/q/$file" )
}
die "Please, add to enviroment SNOWBALL_STEMMER. Exiting...\n"
	if ( !exists $ENV{SNOWBALL_STEMMER} );

my @stemmers;
$|=1;
foreach my $stemdir ( grep { -d $_ } glob $ENV{SNOWBALL_STEMMER}.'/*' ) {
	print "Test directory for stemmer: $stemdir...";
	my $yes=0;
	$stemdir=~s#/\s*$##;
	$stemdir =~ m#^.*/([^/]+)$#;
	my $lang = $1;
	if ( -s $stemdir.'/stem.h' && -s $stemdir.'/stem.c' ) {
		if ( open(HEAD,$stemdir.'/stem.h' ) ) {
			my $head = join(' ', <HEAD>);
			close HEAD;
			$yes = 1 if (
				$head =~ /\b${lang}_create_env\b/ &&
				$head =~ /\b${lang}_close_env\b/ &&
				$head =~ /\b${lang}_stem\b/
			);
		}
	} 
	if ( $yes ) {
		print "Ok\n";
		push @stemmers, $lang;
	} else {
		print "FAILED\n";
	} 
}

if ( $#stemmers < 0 ) {
die "There is no stemmers. Exiting...\n";
}

my $list = join(', ', @stemmers);
print <<EOT;
####################################################
  Using stemmers: $list 
  If it is not desired or list is not full, check stemmer 
  directory ($ENV{SNOWBALL_STEMMER}) and try again to 
  > perl Makefile.PL
####################################################
EOT

#make installation...

my $act = "\n";
open(OUT_H,">config.c") || die "Can't open file for writing";

print OUT_H "/* Autogenerated file!!! */\n";

foreach my $file ( @snowballfiles ) {
	$act .= "$file: $ENV{SNOWBALL_SRC}/q/$file\n\tcp $ENV{SNOWBALL_SRC}/q/$file $file\n";
}

my $commonheaders = join(' ', grep { /\.h$/ } @snowballfiles);

foreach my $lang ( @stemmers ) {
	$act .= "${lang}_stem.h: $ENV{SNOWBALL_STEMMER}/$lang/stem.h\n\tcp $ENV{SNOWBALL_STEMMER}/$lang/stem.h ${lang}_stem.h\n";
	$act .= "${lang}_stem.c: $ENV{SNOWBALL_STEMMER}/$lang/stem.c $commonheaders ${lang}_stem.h\n\tcp $ENV{SNOWBALL_STEMMER}/$lang/stem.c ${lang}_stem.c\n";
	print OUT_H "#include \"${lang}_stem.h\"\n";
}  

print OUT_H "#define NUMSTEM (".($#stemmers+1).")\n";
print OUT_H "STEM stemmers[] = {\n";
foreach my $i ( 0..$#stemmers ) {
	my $lang = $stemmers[$i];
	my $end = ( $i==$#stemmers ) ? '' : ',';
	print OUT_H "   {\"$lang\", NULL, ${lang}_create_env, ${lang}_close_env, ${lang}_stem}$end\n"; 
} 
print OUT_H "};\n";
close OUT_H;

my $copyfile = join( ' ',
	@snowballfiles, 
	map {
		"${_}_stem.h ${_}_stem.c"
	} @stemmers
);

my $objects = join(' ', map { "${_}_stem.o" } @stemmers).' '.
	join(' ', map { $_=~s/\.c$/.o/; $_ } grep { /\.c$/ } @snowballfiles); 


my $headers = join(' ', map { "${_}_stem.h" } @stemmers).' '.$commonheaders;

my $cleanfile = join( ' ', 
	'config.c',
	$copyfile 
);

my $numtest = $#stemmers + 1;
my $err = `sed s#NUMTEST#$numtest# < t/test.t.in > t/test.t`;
if ( $?>>8 ) {
	unlink 't/test.t' if -e 't/test.t';
	print "Problem with sed, 'make test' can't will not proceed\n";
}
 

use ExtUtils::MakeMaker;

WriteMakefile(
        VERSION_FROM   => 'Snowball.pm',
        NAME      => $NAME,
        AUTHOR    => 'Teodor Sigaev <teodor@stack.net>',

        INC       =>'-I.',
        OBJECT    =>"Snowball.o $objects",
	clean	=> { FILES=>"$cleanfile t/test.t" },
	depend    => {
		'Snowball.o' => "config.c $headers",
	},
);

sub MY::postamble{
	return $act;
}

